@@ -1,18 +1,19 @@
-// Copyright (C) 2006-2007 David Sugar, Tycho Softworks.
+// Copyright (C) 2006-2008 David Sugar, Tycho Softworks.
 //
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 3 of the License, or
+// This file is part of GNU ucommon.
+//
+// GNU ucommon is free software: you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as published 
+// by the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
 //
-// This program is distributed in the hope that it will be useful,
+// GNU ucommon is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
-// GNU General Public License for more details.
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
 //
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+// You should have received a copy of the GNU Lesser General Public License
+// along with GNU ucommon.  If not, see <http://www.gnu.org/licenses/>.
 
 /**
  * Thread-aware file system manipulation class.  This is used to provide
@@ -68,6 +69,13 @@ protected:
 #endif
 	int		error;
 
+#ifdef	_MSWINDOWS_
+	static int remapError(void);
+#else
+	inline static int remapError(void)
+		{return errno;};
+#endif
+
 public:
 	/**
 	 * Enumerated file access modes.
@@ -79,7 +87,9 @@ public:
 		ACCESS_RDWR = ACCESS_REWRITE,
 		ACCESS_APPEND,
 		ACCESS_SHARED,
-		ACCESS_DIRECTORY
+		ACCESS_DIRECTORY,
+		ACCESS_STREAM,
+		ACCESS_RANDOM
 	} access_t;
 
 	/**
@@ -123,6 +133,20 @@ public:
 	 */
 	~fsys();
 
+	/**
+	 * Get the descriptor from the object by pointer reference.
+	 * @return low level file handle.
+	 */
+	inline fd_t operator*() const
+		{return fd;};
+
+	/**
+	 * Get the descriptor from the object by casting reference.
+	 * @return low level file handle.
+	 */
+	inline operator fd_t() const
+		{return fd;};
+
 	/**
 	 * Test if file descriptor is open.
 	 * @return true if open.
@@ -168,7 +192,13 @@ public:
 	 * Set the position of a file descriptor.
 	 * @param offset from start of file or "end" to append.
 	 */
-	void	seek(offset_t offset);
+	void seek(offset_t offset);
+
+	/**
+	 * Drop cached data from start of file.
+	 * @param size of region to drop or until end of file.
+	 */
+	void drop(offset_t size = 0);	
 
 	/**
 	 * Read data from descriptor or scan directory.
@@ -287,9 +317,17 @@ public:
 	 * @param descriptor to set.
 	 * @param offset from start of file or "end" to append.
 	 */
-	inline static void seek(fsys& descriptor, size_t offset)
+	inline static void seek(fsys& descriptor, offset_t offset)
 		{descriptor.seek(offset);};
 
+	/**
+	 * Drop cached data from a file descriptor.
+	 * @param descriptor to set.
+	 * @param size of region from start of file to drop or all.
+	 */
+	inline static void drop(fsys& descriptor, offset_t size)
+		{descriptor.drop(size);};
+
 	/**
 	 * Open a file or directory.
 	 * @param path of file to open.
